'
' Your Momma Loves Drama
' Apache Derby - Off to the Races
' Copyright (c) 2005 - George Baklarz, Paul Zikopoulos, Dan Scott
'

' Notes to Users
' Before using this application with Apache Derby, you must catalog the database and the server.
'
' db2 CATALOG TCPIP NODE derby REMOTE localhost SERVER 1527
' db2 CATALOG DATABASE YMLD AT NODE derby AUTHENTICATION SERVER
' db2 catalog system odbc data source ymld
'
Imports Microsoft.Data.Odbc
Public Class YMLD
    '
    '   This class contains the current connection to the database. 
    '
    Private Shared m_conn As OdbcConnection
    Public Shared Property conn() As OdbcConnection
        Get                                                 ' Return the current connection to the database
            Return m_conn
        End Get
        Set(ByVal Value As OdbcConnection)                  ' Create a new connection when you do a set
            m_conn = Value
        End Set
    End Property
End Class

Public Class Display
    '
    '   The class is used to track the forms on the screen. When we close the "purchase ticket" form, we also
    '   want to close all the other windows associated with it. This routine keeps track of the windows that have been
    '   opened, and then allows you to close them all (except the main panel).
    '
    Private Shared current_Forms(10) As Form
    Private Shared current_form As Integer = 0

    Shared Sub addForm(ByVal formin As Form)                ' Add a new form to the collection
        current_form = current_form + 1
        current_Forms(current_form) = formin
    End Sub

    Shared Sub closeForms()                                 ' Close all of the forms except the first one
        Dim i As Integer
        For i = current_form To 1 Step -1
            current_Forms(i).Close()
        Next
        current_form = 0
    End Sub

End Class

Public Class Selection
    '
    '   The Selection class keeps track of what the user has selected on the screen. All of the forms as part of the
    '   purchase ticket transaction keep track of information by using these class variables
    '
    Private Shared start_month(9) As Integer                ' Performance start month
    Private Shared end_month(9) As Integer                  ' Performance end month
    Private Shared titles(9) As String                      ' Title of each performance
    Private Shared current_production As Integer            ' Which performance have I selected?
    Private Shared current_month As Integer                 ' What current month is displayed on the screen
    Private Shared current_day As Integer                   ' What day of a performance have you selected?
    Private Shared current_performance As Integer           ' Current performance number
    Private Shared seats_price(20) As Integer               ' The price of all the seats at a performance
    Private Shared seats_selected(20) As Integer            ' Which seats have you selected for purchase

    Shared Sub setSeat(ByVal seatno As Integer, ByVal selected As Integer, ByVal price As Integer)
        seats_selected(seatno) = selected                   ' Set a seat to occupied/unoccupied, and its price
        seats_price(seatno) = price
    End Sub

    Shared Sub setSeatSelected(ByVal seatno As Integer, ByVal selected As Integer)
        seats_selected(seatno) = selected                   ' Set a set to occupied/unoccupied
    End Sub

    Shared Function getSeatPrice(ByVal seatno As Integer) As Integer
        Return seats_price(seatno)                          ' Get the price for a particular seat
    End Function

    Shared Function getSeatSelected(ByVal seatno As Integer) As Integer
        Return seats_selected(seatno)                       ' Return selected value of seat
    End Function

    Shared Sub clearSeats()                                 ' Clear all seats to unavailable
        Dim i As Integer                                    ' -1=occupied 1=available/selectable 0=occupied/selectable
        For i = 0 To 19
            seats_price(i) = 0
            seats_selected(i) = -1
        Next
    End Sub

    Shared Sub setCurrentProduction(ByVal prodNo As Integer)
        current_production = prodNo                         ' Set the current production number
    End Sub

    Shared Sub setProduction(ByVal prodno As Integer, ByVal startmonth As Integer, ByVal endmonth As Integer, ByVal title As String)
        start_month(prodno) = startmonth                    ' Set production information in the array
        end_month(prodno) = endmonth                        ' Including begin/end month and title information
        titles(prodno) = title
    End Sub

    Shared Function getProduction() As Integer
        Return current_production                           ' Get the current production number
    End Function

    Shared Function getTitle() As String
        Return titles(current_production)                   ' Get the title of the current production
    End Function

    Shared Function getEndMonth() As Integer
        Return end_month(current_production)                ' What is the end month of this production
    End Function

    Shared Function getStartMonth() As Integer
        Return start_month(current_production)              ' What's the start month of this production
    End Function

    Shared Function setCurrentMonth(ByVal whichMonth As Integer)
        current_month = whichMonth                          ' Set the current month of the performance
    End Function

    Shared Function getCurrentMonth() As Integer
        Return current_month                                ' What month are we currently looking at
    End Function

    Shared Sub setPerformance(ByVal perfno As Integer)
        current_performance = perfno                        ' Set the current performance number
    End Sub

    Shared Function getPerformance() As Integer
        Return (current_performance)                        ' Get the current performance number
    End Function

    Shared Function setDay(ByVal dayofmonth As Integer)
        current_day = dayofmonth                            ' Set the day of the performance
    End Function

    Shared Function getDay() As Integer
        Return current_day                                  ' Get the day of the performance
    End Function

End Class

Public Class Productions
    '
    '   Find the production information for a current month.
    '   Given a production number, and a month, this routine will find all of the performances that are taking
    '   place during a month and whether or not any seats are available.
    '
    Private Shared PerfNoDay(32) As Integer

    Shared Function getProdDate(ByVal dayofmonth As Integer) As Integer
        Return PerfNoDay(dayofmonth)                        ' Return the performance number for a particular day
    End Function

    Shared Sub getProdDates(ByVal prodNo As Integer, ByVal prodMonth As Integer)
        Dim myReader As OdbcDataReader
        Dim i As Integer

        For i = 1 To 31                                     ' Zap all dates to no seats available
            PerfNoDay(i) = 0
        Next

        Try                                                 ' Find all performances in a month
            Dim mySelectQuery As String = _
                "SELECT DAY(PERFORMANCE_DATE) AS DAYMONTH, PERFORMANCE_NO " & _
                "FROM PERFORMANCES P " & _
                "WHERE P.PRODUCTION_NO = " & Str(prodNo) & " AND " & _
                "      MONTH(P.PERFORMANCE_DATE) = " & Str(prodMonth) & " AND " & _
                "      P.PERFORMANCE_SEATS > 0 " & _
                "ORDER BY PRODUCTION_NO"

            Dim myCommand As New OdbcCommand(mySelectQuery, YMLD.conn)

            myReader = myCommand.ExecuteReader()
            While myReader.Read()
                Dim dayofmonth = myReader.GetInt32(0)       ' Which day of the month is the performance on
                Dim perfNo = myReader.GetInt32(1)           ' The performance number for this date
                PerfNoDay(dayofmonth) = perfNo              ' Set the array to contain this information
            End While
            myReader.Close()
        Catch se As OdbcException                           ' Errors are not a good thing!
            MsgBox(se.Message, MsgBoxStyle.Exclamation, "YMLD")
        End Try
    End Sub

End Class
